c*********************************************************************
c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/land.F,v $
c $Author: munnich $
c $Revision: 1.6 $
c $Date: 2000/09/11 22:33:35 $
c
c File:  land.f   (sland1, bucket)
c
c Simple-Land(SLand) version 1B           Zeng and Neelin (1998)
c
c Single soil layer (thin (.1m) for Ts; large field capacity for water);
c   a major difference between SLAND1 and the bucket model is that SLAND1 
c   has an evapotranspiration limited by a bulk surface resistance.
c   Other features include: 
c   interception loss is computed using an 'interception function' which
c   incorporates the stochastic property of rainfall in time/space;
c   soft runoff (fast and slow components);
c History:
c   SLand1A:  6/97 for QTCM1 beta version
c   SLand1B:  3/98 for QTCM1 release
c*********************************************************************


           block data landpar
c************
c - properties of land surface as a function of surface type
c     consolidated from BATS, SIB2
c     ocean + 3 land surface types: ocean=0, forest=1, grass=2, desert=3
 
      implicit none
#include "hgrid.h"
#include "qtcm.h"
#include "land.h"
 
c- surface type dependent parameters
      data rsmin/0.,150.,200.,200./   !BATS ocean,trop.forest,tall grass,desert
      data albdveg/.07,.12,.19,.30/   !BATS
      data Z0/.0024,2.,.1,.05/        !BATS
      data xla/0.,6.,3.,1./           !BATS, average for type 2; semidesert 3
c     data vegc/0.,.9,.8,.1/          !BATS, forest,tall grass, semidesert 3
      data WD0/0.,500.,400.,300./     !SIB2/CSU (approximately)
                                      !defined as the part above wilting point

      end
c************************************************


             subroutine sland1
c************
c Simple-LAND(SLAND) version 1B           Zeng and Neelin (1998)
c In the following implementation, sensible heat flux and a 'swamp' evaporation
c   (evaporation like over ocean except using the actual land roughness)
c   are assumed given (calculated in QTCM1 routine [sflux]).
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"
#include "land.h"
 
c --Altered Common Variables:
c Ts,qs,WD,Runf                   over land only
 
c --Local Variables & Functions:
      REAL Rsnet,Fsnet,FWnet
      REAL Evapi,Evapi0,tau_0,Fitc,eps_Ts
      REAL wet,eta,wra,soilC,Runs,Rung,TINY
      REAL Wimax,rints,tau_r,Rung0
      integer Clapp       !@integer for speed; modify if the value is real
      integer i,j,iS

      data TINY/0.0000000001/   !tiny to avoid division by zero

c- non-surface type dependent parameters
      data Wimax/0.1/     !max intercepted water per leaf area; kg/m2 (mm)
      data rints/1.06e-3/ !storm intensity, 3.8mm/hr(from ARME)->mm/s  
      data tau_r/4320./   !storm duration,  1.2hrs ->s
      data Rung0/4.e-4/   !subsurface runoff at saturation; kg/m2(mm)/s
      data Clapp/4/       !Clapp-Hornberger B
      save Wimax,rints,tau_r,Rung0,Clapp

c*****************************************************************
c  time scale to relax Ts to Ts0 at sponge boundary (2 day here)
      eps_Ts=1./(86400.*.2)

c  Soil heat Capacity * density * Depth; assume .1m water-like soil
      soilC=4.18e3*1.e3*.1       !1cal/g/K * 1g/cm3 * .1m;  J/KM2

      do j=1,NY
      do i=1,NX
       iS=STYPE(i,j)
       if(iS.ne.zero) then
 
c= Diagnostics;  surface hydrology
c
c- Interception loss; stochastic rainfall effects included
        Rsnet= FSWds(i,j)-FSWus(i,j)+FLWds(i,j)-FLWus(i,j)  !net sfc. radiation

        Evapi0=Rsnet-FTs(i,j)        !simply set Evapi0 to available energy

        Evapi0=max(TINY,Evapi0)    !minimum of TINY if too small

        tau_0=Wimax*xla(iS)/Evapi0   !time to evaporate a saturated  canopy
     1        *Hlatent               !  in seconds (Evapi0 in W/m2)

        Fitc=(tau_r+tau_0*.8)        !interception function F2 of Zeng (1998)
     &    *Qc(i,j)/(Hlatent*rints*tau_r)        !  Qc in W/m2

        Evapi=Evapi0*Fitc            !interception loss

       arr1(i,j)=Evapi
c---------------------------------------------

c- evaporation; aerodynamic resistance ra from [sflux]
c  stomatal resistance rs=rsmin/wet**alpha with: 
c   alpha=1 - linear dependence 
c   alpha<1 - mimic deep roots water usage such as during dry season  
c   alpha=0 - not limited by water, but by a mimimum stomatal resistance

        wet=WD(i,j)/WD0(iS)           !wetness

        arr3(i,j)=wet

c       wra=one                       !alpha=0 - no wetness dependence
c       wra=wet                       !alpha=1
c       wra=sqrt(wet)                !alpha=1/2
        wra=sqrt(sqrt(wet))         !alpha=1/4

        wra=wra*ra(i,j)               !
        eta=wra/(rsmin(iS)+wra)       !eta=ra/(rs+ra)

        Evap(i,j)=eta*Evap(i,j)       !actual ET = eta * E(swamp)
                                      !E(swamp) was calculated in [sflux]

        Evap(i,j)=Evap(i,j)+Evapi     !total Evap = ET + Interception loss
c---------------------------------------------

c- runoff; note: interception is not available for runoff

        Runs=(Qc(i,j)-Evapi)*wet**4   !surface runoff, BATS formulation

c       Rung=Hlatent*Rung0*wet**(2*Clapp+3)   !subsurface runoff; in W/m2
        Rung=Hlatent*Rung0*(wet**2)**Clapp*wet**3   !optimization for speed

        Runf(i,j)=Runs+Rung           !total runoff; in W/m2

        arr4(i,j)=Runs
c=========================================================
 
c= Prognostic equations
 
cc- soil moisture equation
c
        FWnet=(Qc(i,j)-Evap(i,j)-Runf(i,j))/Hlatent  !net water flux; mm/s

        WD(i,j)=WD(i,j)+dt*FWnet

        WD(i,j)=max(WD(i,j),zero)      !zero if negative (due to numerics)

c- An initially saturated surface will be quickly drained out to a subsaturated
c    wetness (typically .8 for tropical rainforest) by the power law
c    subsurface drainage term; So supersaturation rarely occurs
c       if(WD(i,j)/WD0(iS).gt.1.00) write(*,*) 'warning! wet='
c    &   ,WD(i,j)/WD0(iS),i,j
c---------------------------------------------
 
c- ground temperature; 10cm water-like soil gives a 20min/K damping rate
c    with typical flux; this mimics the surface soil layer response. 
        Fsnet= Rsnet-Evap(i,j)-FTs(i,j)      !net energy absorbed by surface


        Ts(i,j)=Ts(i,j)+dt*Fsnet/soilC 
     &          +dt*spngh0(j)*eps_Ts*(Ts0(i,j)-Ts(i,j))   !relax to Ts0
c=========================================================
       end if
      end do
      end do

!mm      call sump(Ts,ipTs)
!mm      call sump(Runf,ipRunf)
!mm      call sump(Evap,ipEvap)
!mm     call sump(WD,ipWD)

      return
      end


             subroutine bucket
c************
c An implementation of the bucket model (not identical to Manabe et al. 1965).
c Land-surface; Bucket model hydrology except for a 'softer'
c runoff scheme.  May need to be solved implicitly when soil
c depth is thin
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"
#include "land.h"
 
c --Altered Common Variables:
 
c --Local Variables & Functions:
      REAL WD00,wet,soilC,beta
      integer i,j
c*****************************************************************

c- Soil heat Capacity * Depth; assume .1m water-like soil
      soilC=4.18e3*1.e3*.1       !1cal/g/K * 1g/cm3 * .1m;  J/KM2

      do j=1,NY
      do i=1,NX
      if(int(STYPE(i,j)).ne.0) then
cc- hydrology  !W/m**2->mm(depth of water;=kg/m2)/s
        WD00=150.              !field capacity=150mm everywhere
        wet=WD(i,j)/WD00
        Runf(i,j)=wet**4*Qc(i,j)      !depending on P only
        beta=wet
        Evap(i,j)=beta*Evap(i,j)       !actual ET = beta * PotentialE
        WD(i,j)=WD(i,j)+dt*(Qc(i,j)-Evap(i,j)-Runf(i,j))/Hlatent
cc- ground temperature; note conversion factor Cpg
        Ts(i,j)=Ts(i,j)+ dt*(FSWds(i,j)-FSWus(i,j)+FLWds(i,j)
     1            -FLWus(i,j)-Evap(i,j)-FTs(i,j))/soilC
      end if
      end do
      end do
!mm      call sump(Ts,ipTs)
!mm      call sump(Runf,ipRunf)
!mm      call sump(WD,ipWD)
!mm      call sump(Evap,ipEvap)

      return
      end
