c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/qtcm.F,v $
c $Author: munnich $
c $Revision: 1.20 $
c $Date: 2000/10/05 00:59:56 $
c
c File: qtcm.F
c
c This file contains most of the subroutines and functions called by
c   subroutine qtcm, which forms the heart of the Quasi-Equilibrium
c   Tropical Circulation Model (QTCM). The cloud and radiation subroutines
c   are in a separate file and so is the land surface model
c
c Code history:
c   Original (QTCM1 Version 1a):  Ning Zeng, David Neelin, et al. Jan. 1997
c   Beta version (QTCM1 V1b):     June 1997
c   QTCM1 release (QTCMes1 V2.0):   June 1998
c   QTCM1 V2.1 :                  Jan  1999
c
c The following header files are used using INCLUDE:
c   hgrid.h      Horizontal resolution
c   qtcm.h       Primary subroutine common blocks and parameter
c                statements file
c   land.h       Land
c   calendar.h   Variable declarations for virtual calendar
c
c Routines called but not in this file:
c   getSST          in getbnd.F
c   varmean         in output.F
c   everything      in land.F
c   everything      in clrad*.F
c
c File I/O:  There are files remain open throughout (global), and others
c   are closed immediately in the corresponding opening routine before
c   exit (local).
c   Convention: units 20-29 are reserved for global files (always output);
c   units 10-19 for local files (usually input except for qtcm.restart.out);
c   no other units are used.
c   All the output are done in output.F except unit 22 is called from here
c*********************************************************************



             subroutine qtcminit()
c************
c Initialize qtcm atmospheric model variables, datasets, and
c diagnostic fields.
c
      implicit none
c*****************************************************************

      call parinit            !Initialize model parameters
      call varinit            !Initialize variables
      call TimeManager(1)     !mm set model time 
      call bndinit            !input boundary datasets
      call physics1(0,1)      !diagnostic fields for initial condition

      return
      end



             subroutine qtcm(interval)
c************
c Main routine for the Quasi-Equilibrium Tropical Circulation Model.
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"
#include "calendar.h"

c --File I/O:  Unit 22 always open.

c --Arguments:
      integer interval             ! interval of ocean-atmos.
c                                  ! coupling; unaltered on exit

c --Altered Common Variables:
c     REAL Ts                    ! surface temperature

c --Local Variables and Functions:
      integer it
c*****************************************************************

      call getbnd          !get time evolving bndry data; SST, albedo etc.

c- number of atmospheric time steps within one air-sea coupling interval
      nastep = interval* (86400./dt)
c      write (*,*) 'nastep=',nastep
      if(mod(nastep,mt0).ne.0) then
         print*,'qtcm: Fatal error: mt0*dt not a divisor of 1 day'
         stop
      endif

c- land-atmosphere coupling loop
c    land runs at the same timestep as atmo.; easy modification otherwise,
c    but if land runs less often than [sflux] in [physics1], Evap over land 
c    needs to be fixed in [sflux]
      do it=1,nastep
 
          call set_qclock(it)  !advance the diurnal clock
 
          call physics1(it,mt0)   !physics associated with temperature mode T1
 
          call sland1          !land
 
          call atmostep(it,mt0)   !atmospheric dynamics
 
          call varmean(it)         !time mean (sum up variables, output in output.F)

#ifdef CPLMEAN
          call cplmean(it)     !optional mean sfc. flux; for air-sea coupling
#endif
 
c output within the air-sea coupling interval, useful for model diagnosis.
c   you may want to set the user-control parameter ntouti=1 (day)
c         if(dayofmodel.ge.55) call outpinst
c         call outpinst
c         write(*,*) 'it= ',it
      end do

      return
      end



             subroutine parinit
c************
c Initialize physical constants and model parameters
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"
#include "calendar.h"

c --File I/O:  Subroutine opens and closes following files:
c     unit 12 = qtcmpar.in
c   units closed prior to entry, and on exit of the routine
 
c --Local Variables:
      REAL thetau,thetav,sig,y0,yj,y0dfs
      integer i,j
c*****************************************************************
cc- physical/mathematical constants
      Rearth=6.37d6
      rhoair=1.20
      Rair=287.040
      Omega=7.292d-5
      Hlatent=2.43d6
      Cp=1.004d3
      zero=0.0
      one=1.0
      two=2.0
      half=0.50
      pi=asin(one)*two

cc- set up saturation humidity look-up table
      call humtable
#ifndef LINEAR_T1C
cc- set up nonlinear T1c look-up table
      call t1ctable
#endif ! LINEAR_T1C

cc- Model parameters
cc Model physics related parameters
      open(12,file='qtcmpar.in',status='old')
        read(12,*)a1hat
        read(12,*)a1s
        read(12,*)V1s
        read(12,*)b1hat
        read(12,*)b1s
        read(12,*)bb1hat
        read(12,*)GMsr
        read(12,*)GMsp
        read(12,*)GMqr
        read(12,*)GMqp
        read(12,*)GMs0r
        read(12,*)GMq0r
        read(12,*)eps_c
        read(12,*)CV0
        read(12,*)eps_0
        read(12,*)eps_10
        read(12,*)eps_01
        read(12,*)eps_1
        read(12,*)Tsref
        read(12,*)Cpg
        read(12,*)Trefs
        read(12,*)qrefs
        read(12,*)Tcrefs         ! added
        read(12,*)qcrefs         ! added
        read(12,*)Trefhat        ! added
        read(12,*)qrefhat        ! added
        read(12,*)Tcrefhat       ! added
        read(12,*)qcrefhat       ! added
        read(12,*)Vijk
        read(12,*)Vwijk          ! added
        read(12,*)DTijk
        read(12,*)Dqijk
        read(12,*)V1z
      close(12)

c- 
c       GMqp=GMqr*b1s/qrefs     !reset GMqp so qairs/GMq vanish simultaneously
c      write(*,*) 'parinit: GMqp is readjusted to GMqp=',GMqp

c- automatically change numerical viscosity according to the resolution.
      viscx=viscx*NX/64.
      viscy=viscy*NY/32.

c- set up model domain, grid size, geometric factor, Coriolis etc.
      dx=2.*pi*Rearth/NX           !whole globe, periodic in x
      dy=YB/90.*pi*Rearth/NY              !to 90 N/S; but half-grid off pole

       do j=1,NY-1      !for staggered variables along y; pole points not needed
          thetav=dy/Rearth*(j+.5-(NY+1.)/2.)
          fv(j)=2.*Omega*sin(thetav)  !Coriolis parameter at half grid (j+1/2)
          cosv(j)=cos(thetav)         !cos(lat) at half (j+1/2) (staggered)
c         cosv(j)=one                  !turns off spherical geometry
          cosvi(j)=one/cosv(j)         !1/cos(lat) at half grid
      end do

      do j=1,NY        !for centered variables
          thetau=dy/Rearth*(j-(NY+1.)/2.)
          fu(j)=2.*Omega*sin(thetau)  !Coriolis parameter at full grid points
          cosu(j)=cos(thetau)         !cos(lat) at full grid   j
c         cosu(j)=one                  !turns off spherical geometry
          cosui(j)=one/cosu(j)         !1/cos(lat) at full grid

cc-sponge boundary outside y0 N/S with a linear dropoff; with a lower cutoff 
c @Only spngh1 and spngh3 are used currently to reduce excessive bnd. precip. 
          sig=10.0               !in degree latitude
          y0=45.0
          y0dfs=31.0
          yj=thetau*180.0/pi
          spngh0(j)=zero               !mask sponge boundary (for Ts land)
          spngh1(j)=one                !reduce heat/moisture flux
          spngh2(j)=one                !increase diffusion for T,U,V,Q 
          spngh3(j)=one                !increase momentum damping
          spngh4(j)=one                !reduce precipitation Qc only

cc-model results are somewhat sensitive to the use of spngh
cc-ZNC rad1 used spngh0 and spngh1; ZNC rad0 used spngh0,spngh1 and spngh3 
cc-Su et al. used spngh0 and spngh2  
c          if(abs(yj).gt.y0) then
c              spngh0(j)=min(one,(abs(yj)-y0)/sig)
c             spngh0(j)=min(10,(abs(yj)-y0))
c            spngh0(j)=min(15,(abs(yj)-y0))
c             spngh1(j)=max(.5,one-(abs(yj)-y0)/sig)
c             spngh2(j)=one/max(.25,one-(abs(yj)-y0)/sig)
c             spngh3(j)=one/max(.1,one-(abs(yj)-y0)/sig)
c            spngh4(j)=max(.10,one-(abs(yj)-y0)/sig)
c          end if
c          if(abs(yj).gt.y0dfs) then
c            spngh2(j)=one/max(.250,one-(abs(yj)-y0dfs)/sig)
c          endif
      end do
      write(*,*)'parinit: spngh0,1,2,3,4='
     & ,spngh0(1),spngh1(1),spngh2(1),spngh3(1),spngh4(1)

cc- To handle the periodic boundary condition in finite differencing along x, 
c we use a 'double referencing' approach;  the following is the indexing:
      do i=1,NX
          im1(i)=i-1
          ip1(i)=i+1
      end do
      im1(1)=NX
      ip1(NX)=1

      return
      end


             subroutine bndinit
c************
c Time-independent boundary conditions; Initializing time-
c dependent boundaries
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"
#include "land.h"
 
c --File I/O:  Subroutine opens and closes following files:
c     unit 13 = SST01
c     unit 14 = STYPE
c     unit 15 = TOP
c   units closed prior to entry, and on exit of the routine
 
c --Altered Common Variables:
c Ts,CDN,STYPE,ALBDs
 
c --Local Variables & Functions:
      REAL VONKAR,hPBL
      integer i,j
! Functions
      integer lnblnk
c*****************************************************************

cc input  !return the directory name with the trailing blanks removed
      open(13,file=bnddir(1:lnblnk(bnddir))
     &      //'/ALBD_Darnell/00001315.alb',status='old')
      open(14,file=bnddir(1:lnblnk(bnddir))//'/STYPE',status='old')
      open(15,file=bnddir(1:lnblnk(bnddir))//'/TOP',status='old')
      do j=1,NY
      do i=1,NX
          read(13,*) ALBDs(i,j)      !annual mean surface albedo
          read(14,*) STYPE(i,j)      !surface type
          read(15,*) TOP(i,j)        !relative topography, height/10km
      end do
      end do
      close(14)
      close(15)
      

c=  Treat land surface as ocean if land is turned off
      if(landon.ne.1) then
        do j=1,NY
        do i=1,NX
          STYPE(i,j)=zero
        end do
        end do
      end if
      
c=  Make first and last 2 rows ocean points to stabelize the model
      STYPE(1:NX,1:2)=0.
      STYPE(1:NX,NY-1:NY)=0.


c=  Indexing the land points to avoid using 'if' in land.F for vectorization
c     Note: not used in QTCM1V2.1
      nland=0
      do j=1,NY
      do i=1,NX
        if(STYPE(i,j).ne.zero) then
          nland=nland+1
          iland(nland)=i
          jland(nland)=j
        end if
      end do
      end do
      write(*,*) 'bndinit: fraction of land points in model domain='
     1            ,nland*100./NXY,'%'      !only approx. geometry neglected


c= Compute CDN and ALBDs using the assigned surface type
      VONKAR=0.4      !Von Karman constant
      do j=1,NY
      do i=1,NX

c- neutral drag
c  Deardorff (1972) (CCM1 formulation, Williamson et al. 1987; NCAR TN285)
          hPBL=2000.      !2km PBL depth
          CDN(i,j)= 1./
     1      (log(.025*hPBL/Z0(int(STYPE(i,j))))/VONKAR+8.4)**2

c  BATS/CCM2 formulation; CDN larger than CCM1 especially over forest
c         hPBL=75.        !PBL depth
c         CDN(i,j)=(VONKAR/DLOG(hPBL/Z0(int(STYPE(i,j)))))**2


c- surface albedo
c         ALBDs(i,j)=albdveg(int(STYPE(i,j)))   !get albedo from sfc type

      end do
      end do
 
      return
      end



             subroutine varinit
c************
c Initialize model variables.  Will read data for restart run
c if mrestart=1
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"
#include "land.h"
#include "calendar.h"

c --Local Variables & Functions:

 
c --File I/O:  Subroutine opens and closes following files:
c     unit 12 = qtcm.restart
c   units closed prior to entry, and on exit of the routine
 
c --Altered Common Variables:
c all the prognostic variables
 
c --Local Variables & Functions:
      integer idateofmodel
      integer i,j
      character*80 retitle
c*****************************************************************

c- Initialize land points for non-restart run with land on
      do j=1,NY
      do i=1,NX
          Ts(i,j)=295.
          WD(i,j)=WD0(int(STYPE(i,j)))*.7      !70% saturation to start with
c         everything else = zero
      end do
      end do

c- Read in initial condition (all prognostic variables/rhs and a few other 
c    variables for initializing physics) if restart
c @Note: the restart is not perfect at present; we only use it for spin-up

      if(mrestart.eq.1) then
        open(12,file='qtcm.restart'
     1         ,form='unformatted',status='old')
        read(12)idateofmodel,retitle
        read(12)u1,v1,T1,q1                             !baroclinic mode
     1      ,u0,v0,vort0,psi0,rhsvort0,u0bar,rhsu0bar   !barotropic mode
c    2      ,u2,v2                                      !ekman
     3      ,Ts,WD                                      !land
        close(12)
        write(*,*)'varinit: restart the previous run ',retitle,
     1       'at the end of',idateofmodel

!
!  set start date
!
        if(day0.le.0) then
           year0=idateofmodel/10000
           month0=mod(idateofmodel,10000)/100
           day0=mod(idateofmodel,100)+1
           if (monlen(month0).lt. day0) then
              day0=1
              month0=month0+1
              if(month0.gt.12) then
                 month0=1
                 year0=year0+1
              endif
           endif
           write(6,'(a,i4.4,i2.2,i2.2)') 
     1              'Using starting date from restart file:',
     2               year0,month0,day0
        endif

      else
         day0=1
      end if  ! mrestart.eq.1
      

      return
      end



             subroutine out_restart
c************
c Save model variables for restart run. 
c Be sure to save ALL the prognostic variables for atmo and land
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"
#include "land.h"
#include "calendar.h"
 
c --File I/O:  Subroutine opens and closes following files:
c     unit 12 = qtcm.restart.out
c   units closed prior to entry, and on exit of the routine
        
c --Local Variables & Functions:
      character*8 cdate
      character*21 fname
c*****************************************************************

c      write(cdate,'(i8.8)') dateofmodel
c      fname='qtcm_'//cdate//'.restart'
      fname='qtcm.restart.out'
      open(12,file=fname,form='unformatted')
      write(12)dateofmodel,title
      write(12)u1,v1,T1,q1
     1      ,u0,v0,vort0,psi0,rhsvort0,u0bar,rhsu0bar
c    2      ,u2,v2
     3      ,Ts,WD
      close(12)
      write(6,'(a,i8.8)') ' Restart file written at end of ', 
     1                     dateofmodel

      return
      end


             subroutine atmostep(it,mt0)
c************
c Core routine. Time marching, Adams-Bashforth3 scheme in  [barcl],
c [bartr]
c
      implicit none
 
c --local variables
      integer it,mt0  
 
c*****************************************************************
      
      call advctuv       !advection of momentum u1,v1,u0,v0
      
      call advctTq       !advection of T1, q1
      
      call dffus         !diffusion of u1,v1,u0,v0,T1,q1
      
      call barcl         !the baroclinic mode
      
      if(mod(it,mt0).eq.0) call bartr    !barotropic mode

      return
      end



             subroutine physics1(it,mt0)
c************
c Model physics package associated with temperature mode 1:
c   moist convection, cloud, longwave and shortwave radiation,
c   surface fluxes of heat,moisture and momentum
c 
      implicit none

c --local variables
      integer it,mt0

c
c*****************************************************************

      call mconvct         !moist convection; precipitation
      
      call cloud           !cloud prediction
      
      call radsw           !shortwave scheme
      
      call radlw           !longwave radiation scheme

      call sflux           !surface fluxes

c     call other_flux      !e.g., stochastic forcing; add to rhs

      return
      end


#ifndef LINEAR_T1C
!
! default: nonlinear closure
!
             subroutine mconvct
c************
c Moist convection; Betts-Miller scheme projected on temperature mode 1
c 
      implicit none
#include "hgrid.h"
#include "qtcm.h"
#include "calendar.h"

c --Altered Common Variables:
c Qc           !convective heating, also precipitation     W/m**2
 
c --Local Variables & Functions:
      REAL CAPE1
      REAL dTrefhat,dqrefhat,drefs
      REAL T1c(NX,NY)
      integer i,j

      REAL x,nlt1c,T1cr
c*****************************************************************
      dTrefhat=Tcrefhat-Trefhat        ! difference in Tcrhat-Trhat 
      dqrefhat=qcrefhat-qrefhat        ! difference in qcrhat-qrhat
      drefs=Trefs+qrefs-Tcrefs-qcrefs  ! difference in surface refs  

      do j=1,NY
      do i=1,NX
c- calculate projected CAPE using qc closure 1
c         T1c(i,j)=(a1hat*T1(i,j)+b1hat*q1(i,j)
c     &            -dTrefhat-dqrefhat) / (a1hat+bb1hat)
c- calculate projected CAPE using no qc closure
c         T1c(i,j)=a1s*T1(i,j)+b1s*q1(i,j)+drefs
c- calculate projected CAPE using nonlinear T1c table
         x=Trefhat-Tcrefhat+a1hat*T1(i,j)+qrefhat+b1hat*q1(i,j)
         T1c(i,j)=nlt1c(i,j,x)
 
         CAPE1=a1hat*(T1c(i,j)-T1(i,j))+dTrefhat
         CAPE1=max(CAPE1,zero)      !reset to zero if negative
         Qc(i,j)=eps_c*CAPE1          !convective heating
     1            *Cpg*spngh4(j)       !convert to flux W/m^2;sponge bnd.


      end do
      end do

      return
      end


#else   ! linear closure (used prior to V2.2)
             subroutine mconvct
c************
c Moist convection; Betts-Miller scheme projected on temperature mode 1
c 
      implicit none
#include "hgrid.h"
#include "qtcm.h"
#include "calendar.h"

c --Altered Common Variables:
c Qc           !convective heating, also precipitation     W/m**2
 
c --Local Variables & Functions:
      REAL CAPE1
      REAL dTrefhat,dqrefhat,drefs
      REAL T1c(NX,NY)
      integer i,j
c*****************************************************************
      dTrefhat=Tcrefhat-Trefhat        ! difference in Tcrhat-Trhat 
      dqrefhat=qcrefhat-qrefhat        ! difference in qcrhat-qrhat
      drefs=Trefs+qrefs-Tcrefs-qcrefs  ! difference in surface refs     
      do j=1,NY
      do i=1,NX
c- calculate projected CAPE using qc closure 1
         T1c(i,j)=(a1hat*T1(i,j)+b1hat*q1(i,j)
     &            -dTrefhat-dqrefhat) / (a1hat+bb1hat)
c- calculate projected CAPE using no qc closure

         CAPE1=a1hat*(T1c(i,j)-T1(i,j))+dTrefhat
         CAPE1=max(CAPE1,zero)      !reset to zero if negative
         Qc(i,j)=eps_c*CAPE1          !convective heating
     1            *Cpg*spngh4(j)       !convert to flux W/m^2;sponge bnd.


      end do
      end do

      return
      end
#endif ! LINEAR_T1C  end linear conv. 

             subroutine sflux
c************
c Surface fluxes of sensible heat, moisture, and momentum; 
c   saturation humidity, drag
c Note: the wind speed for evaporation/sensible heat is allowed to be
c   computed differently from that for momentum; presently QTCM1 lacks a
c   PBL parameterization so q,T at surface are not simulated accurately enough.
c   This leads to somewhat too sensitive dependence of evaporation on wind
c   speed.  The evaporation-wind feedback, therefore MJO is also quite
c   sensitive to this parameterization.
c 
      implicit none
#include "hgrid.h"
#include "qtcm.h"
#include "land.h"


c --Altered Common Variables:
c CV        !CV*CV0=CD*Vs; drag*velocity       dimensionless
c ra        !ra=one/(CDN(i,j)*VsE(i,j))        aerodynamic resistance, for land
c FTs       !sensible heat                     W/m**2
c Evap      !evaporation                       W/m**2
c taux      !surface stress along x            N/m**2 (=10 dyn/cm**2)
c tauy      !surface stress along y            N/m**2 (=10 dyn/cm**2)
 
c --Local Variables & Functions:
      integer i,j
      REAL Cpi,CV0i
      REAL qs,hsat        ! hsat - function
      REAL rhoCDVs0,Vsmin,Vsmax,VsE(NX,NY),VsM
      REAL eta,alpha,alphai,V1st
      REAL Vst,u1_T(NX,NY),u0_T(NX,NY),v1_T(NX,NY),v0_T(NX,NY)
c*****************************************************************
      Cpi=one/Cp
      CV0i=one/CV0


c-calculate effective surface wind speed for Evaporation and Sensible heat
c A formula is provided for you to 'play' with, e.g.,
c   eta=1,alpha=2 returns the popular formula VsE=sqrt(Vsmin**2+Vs**2)

c     eta=.2
      eta=.6
      alpha=2.0
      alphai=one/alpha
      Vsmin=5.         !a minimum surface wind speed
      Vsmax=8.         !a max surface wind speed; @not used

      do j=1,NY
      do i=1,NX
c actual wind speed
          V1st=-0.17   !900mb winds 
c         V1st=-0.14   !800mb
          u1_T(i,j)=(u1(i,j)+u1(im1(i),j))*0.5
          u0_T(i,j)=(u0(i,j)+u0(im1(i),j))*0.5
          v1_T(i,j)=(v1(i,j)+v1(i,j-1))*0.5
          v0_T(i,j)=(v0(i,j)+v0(i,j-1))*0.5
          Vst=sqrt((u1_T(i,j)*V1st+u0_T(i,j))**2+(v1_T(i,j)*V1st
     &        +v0_T(i,j))**2) 


c a formula is provided for you to play with
c         VsE(i,j)= ( Vsmin**alpha+(eta*Vst)**alpha )**alphai  

c a special case of above with alpha=2; runs faster written this way
          VsE(i,j)=sqrt(Vsmin**2+(eta*Vst)**2)

c a special case of above with alpha=1; runs faster written this way
c         VsE(i,j)=Vsmin+eta*Vst

c yet another formula; linear with low/high cutoff (@ used in beta-version)
c         Vst=max(Vst,Vsmin)  !low cutoff
c         VsE(i,j)=min(Vst,Vsmax)  !high cutoff
      end do
      end do
c-------------------------------------

c- evaporation; pseudo-potential evaporation over land;    W/m^2
      do j=1,NY
      do i=1,NX
          qs=hsat(Ts(i,j))*Hlatent*Cpi    !saturation humidity at surface
          Evap(i,j)=rhoair*Cp*CDN(i,j)    !Cp: q is in Kelvin
     1      *VsE(i,j)*(qs-qrefs-b1s*q1(i,j))
          ra(i,j)=one/(CDN(i,j)*VsE(i,j)) !aerodynamic resistance, pass to land
      end do
      end do
c-------------------------------------

c- sensible heat flux;  W/m^2
      do j=1,NY
      do i=1,NX
          FTs(i,j)=rhoair*Cp*CDN(i,j)*VsE(i,j)      !VsE as for evaporation
     1             *(Ts(i,j)-Trefs-a1s*T1(i,j))
      end do
      end do
c-------------------------------------

c- momentum flux (stress)
c eps_0=gravity/delp*rhoair*CD0*Vs0 defined in par.f where:  CD0=1.4e-3,
c   Vs0=10m/s (values are arbitrary but carried into qtcm by CV0=CD0*Vs0)
c   are used for scaling; the scaling coefficient CV accounts for the
c   actual variation in Vs and CD; here taux/tauy are only diagnostic
c   because the damping is carried by CV one level down to [barcl] and
c   [bartr] as a result of the vertical projection. 
      Vsmin=4.0       !a minimum surface wind speed for momentum 
      do j=1,NY
      do i=1,NX
          VsM=sqrt( (u1_T(i,j)*V1s+u0_T(i,j))**2 
     1             + (v1_T(i,j)*V1s+v0_T(i,j))**2
     2             + Vsmin**2 )
          CV(i,j)=CDN(i,j)*VsM*CV0i            !CV dimensionless
      end do
      end do

      rhoCDVs0=eps_0*Cpg/Cp                   !rhoCDVs0=rhoair*CD0*Vs0
      do j=1,NY
      do i=1,NX
          taux(i,j)=rhoCDVs0*CV(i,j)*(u0_T(i,j)+V1s*u1_T(i,j))
          tauy(i,j)=rhoCDVs0*CV(i,j)*(v0_T(i,j)+V1s*v1_T(i,j))
      end do
      end do

      return
      end


             subroutine barcl
c************
c Baroclinic mode V1; explicit (Euler-backward in a sense)
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"

 
c --Altered Common Variables:
c u1,v1,T1,q1
 
c --Local Variables & Functions:
      REAL dxi,dyi,Cpgi,a1hati,b1hati
      REAL GMs,GMs2,GMq2,uatv,vatu,rhs
      !GMq now as GMq1 in defined in qtcm.h and part of barclcom
      integer i,j
c     integer iop
c 
c*****************************************************************
c save time in the innermost loop
      dxi=one/dx
      dyi=one/dy
      Cpgi=one/Cpg
      a1hati=one/a1hat
      b1hati=one/b1hat

c= x-momentum equation
      do j=1,NY
      do i=1,NX
          vatu=.250*(v1(i,j)+v1(ip1(i),j)+v1(i,j-1)+v1(ip1(i),j-1))

          rhs = -eps_1*spngh3(j)*CV(i,j)*u1(i,j)   !damping
     1    +fu(j)*vatu                         !Coriolis
     2    -Rair*cosui(j)*dxi*(T1(ip1(i),j)-T1(i,j))    !Pressure gradient force
     3    -eps_01*CV(i,j)*u0(i,j)             !barotropic mode
c    4    -eps_21*CV(i,j)*u2(i,j)             !ekman
     5    +advu1(i,j)                         !advection of u
     6    +dfsu1(i,j)                         !diffusion of u

          u1(i,j)=u1(i,j)+dt*rhs
      end do
      end do
c      call xfilter(u1)          !spatial filter at high latitudes
c================================================================

c= y-momentum equation
      do j=1,NY-1
      do i=1,NX
          uatv=.25*(u1(i,j)+u1(im1(i),j)+u1(i,j+1)+u1(im1(i),j+1))

          rhs=-eps_1*spngh3(j)*CV(i,j)*v1(i,j)
     1    -fv(j)*uatv                         !Coriolis
     2    -Rair*dyi*(T1(i,j+1)-T1(i,j))       !pressure gradient force
     3    -eps_01*CV(i,j)*v0(i,j)             !barotropic mode interaction
c    4    -eps_21*CV(i,j)*v2(i,j)             !ekman
     5    +advv1(i,j)                         !advection of v
     6    +dfsv1(i,j)                         !diffusion of v

          v1(i,j)=v1(i,j)+dt*rhs
      end do
      end do

c boundary conditions in y
c pole values are interpolated from points one grid away; these points
c   are in a sense ghost points; note the sign 
c     do i=1,NX/2
c         iop=i+NX/2           !locate the opposite side of the poles
c
c         v1(i,0)=half*(v1(i,1)-v1(iop,1))            !South Pole
c         v1(iop,0)=-v1(i,0)    !point on the other side has opposite sign
c
c         v1(i,NY)=half*(v1(i,NY-1)-v1(iop,NY-1))     !North Pole
c         v1(iop,NY)=-v1(i,NY)
c     end do

c This is not needed if the curvature term is kept.
      do i=1,NX
          v1(i,0)=zero 
          v1(i,NY)=zero
      end do

c      call xfilter(v1)
c================================================================

c= divergence; diagnostic; curvature term neglected
      do j=1,NY
      do i=1,NX
          div1(i,j)=(u1(i,j)-u1(im1(i),j))*cosui(j)*dxi
     1              +(v1(i,j)-v1(i,j-1))*dyi
c         div2(i,j)=(u2(i,j)-u2(im1(i),j))*dxi+(v2(i,j)-v2(i,j-1))*dyi
      end do
      end do

c================================================================

c= Temperature equation
      do j=1,NY
      do i=1,NX
c         GMs=GMsr+GMsp*T1(i,j)  !nonlinear correction to GMs; @not used
          GMs=GMsr
          GMs2=GMs2r
          rhs=
     1       -GMs*div1(i,j)      !adiabatic warming
c    2       -GMs0r*div0(i,j)
c    2       - GMs2*div2(i,j)
     3       +(
     3        +Qc(i,j)           !convective heating
     4        +FSW(i,j)          !shortwave heating
     5        +FLW(i,j)          !longwave heating
     6        +FTs(i,j)          !sensible heat
c    7        +other_flux(i,j)   !other heat fluxes
     8        )*Cpgi*spngh1(j)   !flux -> heating rate; sponge boundary

c Divide by a1hat to account for the vertical averaging/projection
c   note: this effect is already absorbed in adv (through DTijk) and dfs.
          rhs=rhs*a1hati+advT1(i,j)+dfsT1(i,j)
c          rhs=rhs*a1hati+dfsT1(i,j)

          T1(i,j)=T1(i,j)+dt*rhs
      end do
      end do
c      call xfilter(T1)
c================================================================

c= Moisture equation
      do j=1,NY
      do i=1,NX
          GMq1(i,j)=GMqr+GMqp*q1(i,j)  !nonlinear correction to GMq
c         GMq=GMqr
c          GMq2=GMq2r
c          arr2(i,j)=GMq
          rhs= 
     1       +GMq1(i,j)*div1(i,j)      !moisture convergence
c    2       -GMq0r*div0(i,j)
c    3       -GMq2*div2(i,j)
     4       +(
     5        -Qc(i,j)           !convective moistening (drying, of course)
     6        +Evap(i,j)         !moistening due to evaporation
c    7        +other_mflux(i,j)  !other moisture fluxes
     8        )*Cpgi*spngh1(j)   !flux -> heating rate; sponge boundary
          rhs=rhs*b1hati+advq1(i,j)+dfsq1(i,j)  !adv,dfs
c          rhs=rhs*b1hati+dfsq1(i,j)  !adv,dfs

          q1(i,j)=q1(i,j)+dt*rhs

      end do
      end do
c      call xfilter(q1)          !spatial filter at high latitudes
c================================================================

      return 
      end

             subroutine bartr
c************
c Barotropic mode V0
c Non-divergent barotropic vorticity equation; vort0 is the only prognostic
c   variable; streamfunction psi0 is solved with a Poisson solver;
c   then u0, v0 are diagnosed from psi0; time step can be larger than
c   baroclinic mode due to the lack of gravity waves, but be careful with
c   advection
c Note: the dynamic effect of topography is implemented and is found to
c   improve northern hemisphere rainfall and flow pattern significantly.
c   Commented out here. To use it, uncomment fv*div0 in vort0 equation;
c   you can neglect the small contribution to wind field (chi0).
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"

 
c --Altered Common Variables:
c vort0,rhsvort0,u0bar,psi0,u0,v0
 
c --Local Variables & Functions:
      REAL zbar2,zbar                    ! function
      REAL u0CVbar,u1CVbar,u2CVbar,advu0bar,dfsu0bar
      REAL dxi,dyi,V1st,we
      REAL adams1,adams2,adams3
      integer i,j,k,k_1,k_2,ktemp,kw
c     REAL res,res2
c
      data k/1/,k_1/2/,k_2/3/
      save k,k_1,k_2
c*****************************************************************
      adams1=23.0/12.
      adams2=-16.0/12.
      adams3=5.0/12.

      dxi=one/dx
      dyi=one/dy

c= barotropic vorticity equation
c- rhs for bartr
      do j=1,NY-1
      do i=1,NX

c@ use winds at the terrain-following surface (interpolated using V1z)
        we=TOP(i,j)*NZ+one
        kw=we
        we=we-kw
        V1st=(one-we)*V1z(kw)+we*V1z(kw+1)
c       V1st=-0.12    !750mb winds
c       V1st=-0.10    !700mb winds


c divergence generated by topographic lifting; @not used in standard QTCM1 V2.0
c div0 centered at (i+1/2,j+1/2), so an average is done; however, this
c makes it smoother (can potentially make significant difference, at
c large topo. gradient and low resolution).
        div0(i,j)=                       !centered at (i+1/2,j+1/2)
     1   ((u0(i,j)+V1st*u1(i,j))*(TOP(ip1(i),j)-TOP(i,j))
     2     +(u0(i,j+1)+V1st*u1(i,j+1))*(TOP(ip1(i),j+1)-TOP(i,j+1)))
     3    *half*cosvi(j)*dxi
     4  +((v0(i,j)+V1st*v1(i,j))*(TOP(i,j+1)-TOP(i,j))
     5    +(v0(ip1(i),j)+V1st*v1(ip1(i),j))*(TOP(ip1(i),j+1)
     6      -TOP(ip1(i),j)))*half*dyi

        rhsvort0(i,j,k)=
     1   -eps_0*spngh3(j)*(                                            !barcl
     2     (v0(ip1(i),j)*CV(ip1(i),j)-v0(i,j)*CV(i,j))*cosvi(j)*dxi
     3     -(u0(i,j+1)*CV(i,j+1)-u0(i,j)*CV(i,j))*dyi )
     4   -(fu(j+1)-fu(j))*dyi*(v0(i,j)+v0(ip1(i),j))*half              !beta
     5   -eps_10*(                                                     !barcl
     5     (v1(ip1(i),j)*CV(ip1(i),j)-v1(i,j)*CV(i,j))*cosvi(j)*dxi
     6     -(u1(i,j+1)*CV(i,j+1)-u1(i,j)*CV(i,j))*dyi )
c    7   -eps_20*(                                                     !ekman
c    8     (v2(ip1(i),j)*CV(ip1(i),j)-v2(i,j)*CV(i,j))*cosvi(j)*dxi
c    9     -(u2(i,j+1)*CV(i,j+1)-u2(i,j)*CV(i,j))*dyi )
     1   +( (advv0(ip1(i),j)-advv0(i,j))*cosvi(j)*dxi                  !adv
     2     -(advu0(i,j+1)-advu0(i,j))*dyi )
     3   +( (dfsv0(ip1(i),j)-dfsv0(i,j))*cosvi(j)*dxi                  !dfs
     4     -(dfsu0(i,j+1)-dfsu0(i,j))*dyi )
c    5   -fv(j)*div0(i,j)                             !diverg. due to TOPO

        vort0(i,j)=vort0(i,j)+dt*mt0*(adams1*rhsvort0(i,j,k)
     1      +adams2*rhsvort0(i,j,k_1)+adams3*rhsvort0(i,j,k_2))
      end do
      end do
c      call xfilter(vort0)
c==================================================================

c= solve for the divergent part; only a small contribution to the total wind
c     call pois2(dx,dy,div0(1,1),chi0)         ! Poisson solver
c==================================================================

c= advance u0bar(u0 domain average); needed as a bnd. condition for psi0
      u0CVbar=zbar2(u0(1,1),CV(1,1),NXY)
      u1CVbar=zbar2(u1(1,1),CV(1,1),NXY)     !CV maybe a function of space
c      u2CVbar=zbar2(u2(1,1),CV(1,1),NXY)
      advu0bar=zbar(advu0(1,1),NXY)
      dfsu0bar=zbar(dfsu0(1,1),NXY)
      rhsu0bar(k)=
     1    -eps_0*u0CVbar
     2    -eps_10*u1CVbar
c    3    -eps_20*u2CVbar
     4    +advu0bar
     5    +dfsu0bar
      u0bar=u0bar+dt*mt0*(adams1*rhsu0bar(k)
     1    +adams2*rhsu0bar(k_1)+adams3*rhsu0bar(k_2))

c= solve for streamfunction psi0 by inverting Laplacian
c y boundary: v=0 => psi0=const.    (no x dependence)
c                        =-u0bar*Dy at north (set psi0(south)=0)
      do i=1,NX
          psi0(i,NY)=-NY*dy*u0bar     !not NY-1 due to the staggered grid
          psi0(i,0)=zero
      end do

      call fatdfe(dx,dy,cosu,cosv,vort0(1,1),psi0(1,0))    !poisson solver

c check residual; Note: curvature term has been neglected (Holton 1992).
c     res=zero
c     res2=zero
c     do j=1,NY-1
c     do i=1,NX
c       res=res+abs(vort0(i,j)
c    1    - (psi0(ip1(i),j)-2.0*psi0(i,j)+psi0(im1(i),j))
c    2      *dxi*dxi*cosvi(j)*cosvi(j)
c    3    - (psi0(i,j+1)-2.0*psi0(i,j)+psi0(i,j-1))
c    4      *dyi*dyi)                  !curvature term neglected
c       res2=res2+abs(vort0(i,j))
c     end do
c     end do
c     write(*,*)'res,res/rhs',res,res/res2
c==================================================================


c= calculate u0,v0
      do j=1,NY
      do i=1,NX
          u0(i,j)=(psi0(i,j-1)-psi0(i,j))*dyi
c    1           +(chi0(ip1(i),j)-chi0(i,j))*dxi  !modify for cos factor if used
          v0(i,j)=cosvi(j)*(psi0(i,j)-psi0(im1(i),j))*dxi
c    1           +(chi0(i,j+1)-chi0(i,j))*dyi
      end do
      end do

c force the boundary condition
      do i=1,NX
          v0(i,0)=zero
          v0(i,NY)=zero
      end do
c==================================================================

c summation for output mean

c shift the pointers for rhs by one timestep
      ktemp=k_2
      k_2=k_1
      k_1=k
      k=ktemp

      return
      end


             subroutine advctuv
c************
c Advection for momentum u,v,u0,v0, ...
c Centered second order differencing except duy at N/S boundaries
c
c Array adv* is used to pass variables.  This is useful for
c   the barotropic mode where the curl of advection is needed later
c   (because it is solved in streamfunction-vorticity formulation).
c Otherwise the procedure can be applied pointwise without saving
c   in adv*.  Since memory is not a major concern here we do
c   this for diffusion as well as advection/diffusion for other
c   modes.  This always looks neater.
c Note the three-level looping due to the nonlinear advection and
c   projection.
c Moderately optimized for speed
c
      implicit none
#include "hgrid.h"
#include "qtcm.h"

 
c --Altered Common Variables:
c advu1,advv1,advu0,advv0
 
c --Local Variables & Functions:
      REAL u1atu,v1atu,u1atv,v1atv
      REAL u0atu,v0atu,u0atv,v0atv
      REAL du1xdxi,du1ydyi,dv1xdxi,dv1ydyi
      REAL du0xdxi,du0ydyi,dv0xdxi,dv0ydyi                   
      REAL dxi,dyi,ajp1(NY),ajm1(NY),aj(NY)
      REAL vdv1,vdv2,vdv3,vdv4,Vijkt(8)
      REAL divv1,divv2,divv3,divv4,Vwijkt(8)
      REAL cjdxih,dxih,dyih ! mm speed
      integer i,j
      equivalence(Vijk(0,0,0),Vijkt(1))
      equivalence(Vwijk(0,0,0),Vwijkt(1))
      
c*****************************************************************
      dxi=one/dx
      dyi=one/dy
      dxih=half*dxi 
      dyih=half*dyi 

cc- handle 1st-order differencing for duy at y boundaries
      do j=2,NY-1            !centered differencing in interior
          ajp1(j)=half       
          aj(j)=zero
          ajm1(j)=-half
      end do
      ajp1(1)=one             !1st order differencing at bnd.
      aj(1)=-one
      ajm1(1)=zero
      ajp1(NY)=zero
      aj(NY)=one
      ajm1(NY)=-one

c=  x momentum equation; advection for u1, u0

      do j=1,NY
         cjdxih=cosui(j)*dxih
      do i=1,NX
c mode V1
          du1xdxi=(u1(ip1(i),j)-u1(im1(i),j))*cjdxih       !centered differencing
          du1ydyi=(ajp1(j)*u1(i,j+1)+aj(j)*u1(i,j)
     *         +ajm1(j)*u1(i,j-1))*dyi
          u1atu=u1(i,j)
          v1atu=.250*(v1(i,j)+v1(ip1(i),j)+v1(i,j-1)
     *                     +v1(ip1(i),j-1))
c mode V0
          du0xdxi=(u0(ip1(i),j)-u0(im1(i),j))*cjdxih       !centered differencing
          du0ydyi=(ajp1(j)*u0(i,j+1)+aj(j)*u0(i,j)
     *         +ajm1(j)*u0(i,j-1))*dyi
          u0atu=u0(i,j)
          v0atu=.250*(v0(i,j)+v0(ip1(i),j)+v0(i,j-1)
     *                     +v0(ip1(i),j-1))
c
c Loop for different modes; explicit summation makes the routine [advctuv]
c   more than twice as fast. However, looping is easier to implement
c   if you need to add more modes (simply change the parameter value NVMOD).
c       do k=0,NVMOD-1
c         advu(k)=zero
c         do n=0,NVMOD-1
c         do m=0,NVMOD-1
c           advu(k)=advu(k)-Vijk(m,n,k)
c    1          *(uatu(m)*dux(n)*dxi+vatu(m)*duy(n)*dyi)
c    2          -Vwijk(m,n,k)*(div(m)*uatu(n)) !symbolic form for vrt advection 
c         end do
c         end do
c       end do

          vdv1=u0atu*du0xdxi+v0atu*du0ydyi
          vdv2=u1atu*du0xdxi+v1atu*du0ydyi
          vdv3=u0atu*du1xdxi+v0atu*du1ydyi
          vdv4=u1atu*du1xdxi+v1atu*du1ydyi

c= divergence; curvature term neglected
c          div1(i,j)=(u1(i,j)-u1(im1(i),j))*cosui(j)*dxi
c     1              +(v1(i,j)-v1(i,j-1))*dyi

c          divv1=div0(i,j)*u0atu               !div0 is 0   
c          divv2=div1(i,j)*u0atu      
c          divv3=div0(i,j)*u1atu               !div0 is 0 
c          divv4=div1(i,j)*u1atu 

          advu0(i,j)=
     *      -Vijkt(1)*vdv1
     *      -Vijkt(2)*vdv2
     *      -Vijkt(3)*vdv3
     *      -Vijkt(4)*vdv4
c     *      -Vwijkt(1)*divv1
c     *      -Vwijkt(2)*divv2
c     *      -Vwijkt(3)*divv3
c     *      -Vwijkt(4)*divv4
c    1      -Vijk(0,0,0)*vdv1    !this is about 30% slower than above
c    2      -Vijk(1,0,0)*vdv2
c    3      -Vijk(0,1,0)*vdv3
c    4      -Vijk(1,1,0)*vdv4
c    1      -Vwijk(0,0,0)*divv1
c    2      -Vwijk(1,0,0)*divv2
c    3      -Vwijk(0,1,0)*divv3
c    4      -Vwijk(1,1,0)*divv4

          advu1(i,j)=
     *      -Vijkt(5)*vdv1
     *      -Vijkt(6)*vdv2
     *      -Vijkt(7)*vdv3
     *      -Vijkt(8)*vdv4
c     *      -Vwijkt(5)*divv1
c     *      -Vwijkt(6)*divv2
c     *      -Vwijkt(7)*divv3
c     *      -Vwijkt(8)*divv4


      end do
      end do
c====================================================================


c=  y momentum equation; advection for v0, v1
      do j=1,NY-1
         cjdxih=cosvi(j)*dxih
      do i=1,NX
c mode V1
          dv1xdxi=(v1(ip1(i),j)-v1(im1(i),j))*cjdxih
          dv1ydyi=(v1(i,j+1)-v1(i,j-1))*dyih
          u1atv=.250*(u1(i,j)+u1(im1(i),j)+u1(i,j+1)
     *                     +u1(im1(i),j+1))
          v1atv=v1(i,j)
c mode V0
          dv0xdxi=(v0(ip1(i),j)-v0(im1(i),j))*cjdxih
          dv0ydyi=(v0(i,j+1)-v0(i,j-1))*dyih
          u0atv=.250*(u0(i,j)+u0(im1(i),j)+u0(i,j+1)
     *                     +u0(im1(i),j+1))
          v0atv=v0(i,j)

          vdv1=u0atv*dv0xdxi+v0atv*dv0ydyi
          vdv2=u1atv*dv0xdxi+v1atv*dv0ydyi
          vdv3=u0atv*dv1xdxi+v0atv*dv1ydyi
          vdv4=u1atv*dv1xdxi+v1atv*dv1ydyi

c          divv1=div0(i,j)*v0atv               !div0 is 0
c          divv2=div1(i,j)*v0atv
c          divv3=div0(i,j)*v1atv               !div0 is 0
c          divv4=div1(i,j)*v1atv


          advv0(i,j)=
     *      -Vijkt(1)*vdv1
     *      -Vijkt(2)*vdv2
     *      -Vijkt(3)*vdv3
     *      -Vijkt(4)*vdv4
c     *      -Vwijkt(1)*divv1
c     *      -Vwijkt(2)*divv2
c     *      -Vwijkt(3)*divv3
c     *      -Vwijkt(4)*divv4


          advv1(i,j)=
     *      -Vijkt(5)*vdv1
     *      -Vijkt(6)*vdv2
     *      -Vijkt(7)*vdv3
     *      -Vijkt(8)*vdv4
c     *      -Vwijkt(5)*divv1
c     *      -Vwijkt(6)*divv2
c     *      -Vwijkt(7)*divv3
c     *      -Vwijkt(8)*divv4

      end do
      end do
c====================================================================
 

      return
      end


             subroutine advctTq
c************
c Advection of T and q; structured to allow more modes in the future
c Moderately optimized for speed
c
      implicit none
#include "hgrid.h"
#include "qtcm.h"

 
c --Altered Common Variables:
c adv
 
c --Local Variables & Functions:
      REAL u0atC,v0atC,u1atC,v1atC
      REAL dT1x,dT1y,dq1x,dq1y
      REAL dxi,dyi,ajp1(NY),ajm1(NY),aj(NY)
      integer i,j
      REAL DTijkt(2),Dqijkt(2)
      equivalence(DTijk(0,1,1),DTijkt(1))
      equivalence(Dqijk(0,1,1),Dqijkt(1))
c*****************************************************************

      dxi=one/dx
      dyi=one/dy

cc- handle 1st-order differencing for dTy,dqy at y boundaries
      do j=2,NY-1            !centered differencing in interior
          ajp1(j)=half       
          aj(j)=zero
          ajm1(j)=-half
      end do
      ajp1(1)=one             !1st order differencing at bnd.
      aj(1)=-one
      ajm1(1)=zero
      ajp1(NY)=zero
      aj(NY)=one
      ajm1(NY)=-one

      do j=1,NY
      do i=1,NX
c advection due to mode V1
          u1atC=(u1(i,j)+u1(im1(i),j))*half
          v1atC=(v1(i,j)+v1(i,j-1))*half
c mode V0
          u0atC=(u0(i,j)+u0(im1(i),j))*half
          v0atC=(v0(i,j)+v0(i,j-1))*half

c dTx,dTy; centered differencing
          dT1x=(T1(ip1(i),j)-T1(im1(i),j))*half
          dT1y=ajp1(j)*T1(i,j+1)+aj(j)*T1(i,j)
     1         +ajm1(j)*T1(i,j-1)

c dqx,dqy
          dq1x=(q1(ip1(i),j)-q1(im1(i),j))*half
          dq1y=ajp1(j)*q1(i,j+1)+aj(j)*q1(i,j)
     1         +ajm1(j)*q1(i,j-1)

c sum up contributions from different modes 
c       do k=1,NTMOD
c         advT(k)=zero
c         advq(k)=zero
c         end do
c         do n=1,NTMOD
c         do m=0,NVMOD-1
c             advT(k)=advT(k)-DTijk(m,n,k)
c    1               *(uatC(m)*dTx(n)*dxi+vatC(m)*dTy(n)*dyi)
c             advq(k)=advq(k)-Dqijk(m,n,k)
c    1               *(uatC(m)*dqx(n)*dxi+vatC(m)*dqy(n)*dyi)
c         end do
c         end do
c       end do

          advT1(i,j)=
     1    -DTijkt(1)*(cosui(j)*u0atC*dT1x*dxi+v0atC*dT1y*dyi)
     2    -DTijkt(2)*(cosui(j)*u1atC*dT1x*dxi+v1atC*dT1y*dyi)
c    1    -DTijk(0,1,1)*(cosui(j)*u0atC*dT1x*dxi+v0atC*dT1y*dyi)
c    2    -DTijk(1,1,1)*(cosui(j)*u1atC*dT1x*dxi+v1atC*dT1y*dyi)
          advq1(i,j)=
     1    -Dqijkt(1)*(cosui(j)*u0atC*dq1x*dxi+v0atC*dq1y*dyi)
     2    -Dqijkt(2)*(cosui(j)*u1atC*dq1x*dxi+v1atC*dq1y*dyi)

      end do
      end do

      return
      end



             subroutine dffus
c************
c Diffusion for u,v,u0,v0,T1,q1
c Arrays dfs* are used to pass variables. 
c N/S boundary: flux zero (handled by dj(),...)
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "qtcm.h"

 
c --Altered Common Variables:
c dfsu1,dfsv1,dfsu0,dfsv0,dfsT1,dfsq1
 
c --Local Variables & Functions:
      REAL dx2i,dy2i,d2px,d2py,djm(NY),djp(NY)
      integer i,j
c*****************************************************************

      dx2i=one/dx**2
      dy2i=one/dy**2

cc- handle flux zero condition for duy,dTy,dqy at y boundaries
      do j=1,NY
          djm(j)=one    !at j-1/2
          djp(j)=one    !at j+1/2
      end do
      djm(1)=zero       !flux zero at southern bnd
      djp(NY)=zero      !flux zero at northern bnd
 
c= u1 
      do j=1,NY
      do i=1,NX
          d2px=u1(ip1(i),j)-two*u1(i,j)+u1(im1(i),j)
          d2py=djp(j)*u1(i,j+1)
     1        -(djp(j)+djm(j))*u1(i,j)
     2        +djm(j)*u1(i,j-1)
          dfsu1(i,j)=spngh2(j)
     1        *(viscx*d2px*cosui(j)**2*dx2i+viscy*d2py*dy2i)
      end do
      end do

c= v1 
      do j=1,NY-1
      do i=1,NX
          d2px=v1(ip1(i),j)-two*v1(i,j)+v1(im1(i),j)
          d2py=v1(i,j+1)-two*v1(i,j)+v1(i,j-1)
          dfsv1(i,j)=spngh2(j)
     1        *(viscx*d2px*cosvi(j)**2*dx2i+viscy*d2py*dy2i)
      end do
      end do

c= u0 
      do j=1,NY
      do i=1,NX
          d2px=u0(ip1(i),j)-two*u0(i,j)+u0(im1(i),j)
          d2py=djp(j)*u0(i,j+1)
     1        -(djp(j)+djm(j))*u0(i,j)
     2        +djm(j)*u0(i,j-1)
          dfsu0(i,j)=spngh2(j)
     1        *(viscx*d2px*cosui(j)**2*dx2i+viscy*d2py*dy2i)
      end do
      end do

c= v0 
      do j=1,NY-1
      do i=1,NX
          d2px=v0(ip1(i),j)-two*v0(i,j)+v0(im1(i),j)
          d2py=v0(i,j+1)-two*v0(i,j)+v0(i,j-1)
          dfsv0(i,j)=spngh2(j)
     1        *(viscx*d2px*cosvi(j)**2*dx2i+viscy*d2py*dy2i)
      end do
      end do

c= T1
      do j=1,NY
      do i=1,NX
          d2px=T1(ip1(i),j)-two*T1(i,j)+T1(im1(i),j)
          d2py=djp(j)*T1(i,j+1)
     1        -(djp(j)+djm(j))*T1(i,j)
     2        +djm(j)*T1(i,j-1)
          dfsT1(i,j)=spngh2(j)
     1        *(viscx*d2px*cosui(j)**2*dx2i+viscy*d2py*dy2i)
      end do
      end do

c= q1
      do j=1,NY
      do i=1,NX
          d2px=q1(ip1(i),j)-two*q1(i,j)+q1(im1(i),j)
          d2py=djp(j)*q1(i,j+1)
     1        -(djp(j)+djm(j))*q1(i,j)
     2        +djm(j)*q1(i,j-1)
          dfsq1(i,j)=spngh2(j)
     1        *(viscxq*d2px*cosui(j)**2*dx2i+viscyq*d2py*dy2i)
      end do
      end do

      return
      end


! ==============================================================================
      subroutine fatdfe(dx,dy,cosu,cosv,rhs,u)
!
! Front-end for the FATD solver
! 
!! Solves 2-dimensional, second-order, periodic (in X) elliptic
!! problem with coefficients given as functions of Y (constant in X).
!! with periodicity in X s.t.  u(0,j) <- u(nx,j)  and  u(nx+1,j) <- u(1,j)
c  Spherical geometry and Dirichlet boundary conditions in Y.
!
! Arguments:
!  integer         dx,dy        - grid spacing (unchanged)
!  real(NX,NY)     rhs        - vorticity (unchanged); undefined at rhs(i,NY)
!  real(NX,0:NY) u      - stream function (returned)
!                         u(i,0) is used as the southern B.C. (unchanged)
!                         u(i,NY) is used as the northern B.C. (unchanged)
!
! written: Mar 24, 1997  A.J.Adcroft, UCLA (adcroft@atmos.ucla.edu)
! modified: to accomodate the spherical geometry    4/98  N.Zeng
!
      implicit none
! Common
#include "hgrid.h"
! Arguments
      REAL dx,dy
      REAL rhs(nx,ny)
      REAL u(nx,0:ny)
      REAL cosu(ny),cosv(0:ny)
! Local
      integer i,j,nym1
      REAL dx2,dy2i,dxc2
      REAL a(ny-1),b(ny-1),c(ny-1),d(ny-1)
      REAL bet(nx,ny-1),gam(nx,ny-1)
      REAL wsave(2*(nx)+15,ny-1)
      REAL r(nx,ny-1),uu(nx,ny-1)
      logical firstcall
      data firstcall/.true./
      save nym1,dx2,dy2i,a,bet,gam,wsave,firstcall
! ------------------------------------------------------------------------------

      if (firstcall) then
c      print *,'Doing FATD initialisation'
       nym1=ny-1
       dx2=dx**2
       dy2i=1.0/dy**2
       do j=1,nym1
        a(j)=dy2i*cosv(j)**2*dx2
        b(j)=-2.0-2.0*dy2i*cosv(j)**2*dx2
        c(j)=dy2i*cosv(j)**2*dx2
        d(j)=1.0
       enddo

c Dirichlet boundary conditions: u=const.
       a(1)=0.
       c(nym1)=0.

       call inifatd(nx,nym1,a,b,c,d,bet,gam,wsave)
c      print *,'fatdpkg: Finished FATD initialisation'
       firstcall=.false.
      endif

      do j=1,nym1
       dxc2=dx2*cosv(j)**2
       do i=1,nx
        r(i,j)=rhs(i,j)*dxc2     !multiply rhs through by (dx*cos)**2
       enddo
      enddo

      do i=1,nx            !mv bnd values to rhs
       r(i,1)=r(i,1)-cosv(1)**2*dx2*dy2i*u(i,0)
       r(i,nym1)=r(i,nym1)-cosv(nym1)**2*dx2*dy2i*u(i,NY)
      enddo

      call fatd(nx,nym1,a,bet,gam,wsave,r,uu)
!     print *,'Returned from FATD'

      do j=1,nym1
       do i=1,nx
        u(i,j)=uu(i,j)
       enddo
      enddo

      return
      end



             subroutine xfilter(x)
c************
c spatial filter of high wave numbers at high latitudes; 
c   using Arakawa/Lamb filter and FFTPACK (shared with the poisson solver
c   of Adcroft) 
c
      implicit none
#include "hgrid.h"
#include "qtcm.h" 
 
c --Altered Variables:
c x
 
c --Local Variables & Functions:
      REAL wsave(2*NX+15),x(NX,NY),ara(NX,NY)
      integer i,j,js,jn,m
      logical firstcall
      data firstcall/.true./
      save ara,firstcall,js,jn
c*****************************************************************
      if(firstcall) then
          call rffti(NX,wsave)       !initialize FFTPACK

          do j=1,NY                  !low-pass filter of Arakawa/Lamb(1977);
          do i=2,NX                  !  no filtering on wave-number zero(i=1);
            m=i/2                    !m=wave number
            ara(i,j)=cosu(j)/sin(m*pi/NX)*.9     !a reduction of .9
            ara(i,j)=min(ara(i,j),one)
          end do
          end do

          js=(1.-45./YB)*NY/2           !apply the filter outside 45 N/S
          jn=NY-js+1

          firstcall=.false.
      endif


      do j=1,NY
c       if(j.eq.js+1) j=jn           !redundant without this; but not costly

        call rfftf(NX,x(1,j),wsave)  !Fourier transform; rfftf returns in order
                                     ! of increasing wave number: 0,1,1,2,2,...
        do i=2,NX
            x(i,j)=x(i,j)*ara(i,j)   !filtering; not on wavenumber zero
        end do

        call rfftb(NX,x(1,j),wsave)  !transform back
      end do

      return
      end


c************************************************************
c            End of qtcm.F file
c************************************************************

